/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#include "test.h"
#include <mico/dynany_impl.h>


CORBA::ORB_var orb;
CORBA::BOA_var boa;
DynamicAny::DynAnyFactory_var factory;


void dyn_union ()
{
    DynamicAny::DynUnion_var da =
        factory->create_dyn_union (_tc_Union);

    DynamicAny::DynEnum_var disc = DynamicAny::DynEnum::_narrow (
        da->discriminator());
    assert (!CORBA::is_nil (disc));

    da->set_as_default (TRUE);
    assert (disc->value_as_ulong() == 2);

    assert (da->seek (1));
    DynamicAny::DynAny_var value = da->current_component();
    value->insert_short (10);

    da->set_as_default (FALSE);
    assert (disc->value_as_ulong() != 2);

    assert (da->seek (1));
    disc->value_as_string ("el2");
    da->insert_boolean (TRUE);

    da->destroy ();
}

void dyn_seq ()
{
    DynamicAny::DynSequence_var da =
        factory->create_dyn_sequence (_tc_Sequence);
    da->length (10);

    for (int i = 0; i < 10; ++i) {
        da->insert_short (i);
    }

    DynamicAny::AnySeq_var seq = da->get_elements();
    for (int i = 0; i < 10; ++i) {
        CORBA::Short s;
        assert (seq[i] >>= s);
        assert (s == i);
    }

    da->destroy ();
}

void dyn_enum ()
{
    DynamicAny::DynEnum_var da = factory->create_dyn_enum (_tc_Enum);
    da->value_as_string ("el1");
    assert (da->value_as_ulong() == 0);

    da->value_as_ulong (1);
    CORBA::String_var str = da->value_as_string ();
    assert (!strcmp (str, "el2"));

    da->destroy ();
}

void dyn_struct ()
{
    CORBA::Any a;
    Struct s;
    s.l = 10;
    s.s = (const char *)"foobar";
    a <<= s;
    
    DynamicAny::DynAny_var da = factory->create_dyn_any (a);
    assert (da->get_long () == 10);
    CORBA::String_var str = da->get_string ();
    assert (!strcmp (str, "foobar"));

    CORBA::Any_var av = da->to_any ();
    assert (a == av);

    da->destroy ();
}

void dyn_basic ()
{
    DynamicAny::DynAny_var da = factory->create_basic_dyn_any(CORBA::_tc_long);
    da->insert_long (10);

    assert(da->get_long() == 10);

    da->destroy ();
}

void main (int argc, char *argv[])
{
    // ORB initialization
    orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
    boa = orb->BOA_init (argc, argv, "mico-local-boa");
    factory = new DynAnyFactory_impl;

    dyn_basic ();
    dyn_struct ();
    dyn_enum ();
    dyn_seq ();
    dyn_union ();
    return 0;
}
