/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#include <iostream.h>
#include <fstream.h>
#include "grid.h"


int main( int argc, char *argv[] )
{
  // ORB initialization
  CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );

  CORBA::String_var ref;

  ifstream istr( "/tmp/gridiiop.ref" );
  if( !istr ) {
    cerr << "error: file /tmp/gridiiop.ref not readable" << endl;
    exit( 1 );
  }
  char buf[ 1024 ];
  istr >> buf;
  ref = (const char *) buf;
  
  CORBA::Object_var obj = orb->string_to_object( ref );
  assert( !CORBA::is_nil( obj ) );
  
  grid_var client = grid::_narrow( obj );
  assert( !CORBA::is_nil( client ) );
  
  for( int i = 0; i < client->height(); i++ )
    for( int j = 0; j < client->width(); j++ )
      client->set( i, j, i * j );
  
  for( int i = 0; i < client->height(); i++ )
    for( int j = 0; j < client->width(); j++ )
      cout << "[" << i << "," << j << "] = " << client->get( i, j ) << endl;
}
