/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#include "account.h"
#include <mico/naming.h>

int main( int argc, char *argv[] )
{
  // ORB initialization
  CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
  CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");

  CORBA::Object_var nobj =
    orb->resolve_initial_references ("NameService");
  assert (! CORBA::is_nil (nobj));
  
  CosNaming::NamingContext_var nc = CosNaming::NamingContext::_narrow (nobj);
  CosNaming::Name name;
  name.length (1);
  name[0].id = CORBA::string_dup ("myAccount");
  name[0].kind = CORBA::string_dup ("");
  
  CORBA::Object_var obj;

#ifdef HAVE_EXCEPTIONS
  try {
    obj = nc->resolve (name);
  }
  catch (CosNaming::NamingContext::NotFound_var &exc) {
    cerr << "NotFound exception" << endl;
  }
  catch (CosNaming::NamingContext::CannotProceed_var &exc) {
    cerr << "CannotProceed exception" << endl;
  }
  catch (CosNaming::NamingContext::InvalidName_var &exc) {
    cerr << "InvalidName exception" << endl;
  }
#else
  obj = nc->resolve (name);
#endif
  Account_var client = Account::_narrow( obj );

  client->deposit( 100 );
  client->deposit( 100 );
  client->deposit( 100 );
  client->deposit( 100 );
  client->deposit( 100 );
  client->withdraw( 240 );
  client->withdraw( 10 );
  cout << "Balance is " << client->balance() << endl;
  
  return 0;
}
