// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __corba_h__
#define __corba_h__


/***************************** C headers ****************************/

#include <mico/rtti.h>
#include <mico/config.h>
#include <mico/assert.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif


/***************************** STL headers **************************/


#ifdef HAVE_MINI_STL
#include <ministl/string>
#include <ministl/vector>
#include <ministl/map>
#include <ministl/set>
#include <ministl/list>
#else
#include <string>
#include <vector>
#include <map>
#include <set>
#include <list>
#endif

/************************** MICO templates **************************/

#include <mico/version.h>
#include <mico/types.h>
#include <mico/sequence.h>
#include <mico/array.h>
#include <mico/var.h>


/*************************** module CORBA ***************************/

#ifdef HAVE_NAMESPACE
#define MICO_NAMESPACE_DECL namespace
#define MICO_EXPORT_DECL extern
#else
#define MICO_NAMESPACE_DECL struct
#define MICO_EXPORT_DECL static
#endif


#define MICO_NO_TOPLEVEL_MODULES

MICO_NAMESPACE_DECL CORBA {
#define MICO_MODULE_CORBA
#include <mico/basic.h>
#include <mico/address.h>
#include <mico/ior.h>
#include <mico/transport.h>
#include <mico/shlib.h>
#include <mico/process.h>
#include <mico/buffer.h>
#include <mico/codec.h>
#include <mico/string.h>
#include <mico/except.h>
#include <mico/any.h>
#include <mico/object.h>
#include <mico/boa.h>
#include <mico/orb_fwd.h>
#include <mico/dii.h>
#include <mico/dsi.h>
#include <mico/dispatch.h>
#include <mico/ir.h>
#include <mico/typecode.h>
#include <mico/orb.h>
#include <mico/ioptypes.h>
#include <mico/imr.h>
#undef MICO_MODULE_CORBA
};

#define MICO_MODULE__GLOBAL
#include <mico/dispatch.h>
#include <mico/ir.h>
#include <mico/imr.h>
#undef MICO_MODULE__GLOBAL

#undef MICO_NO_TOPLEVEL_MODULES

#endif // __corba_h__
