// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_address_impl_h__
#define __mico_address_impl_h__


class LocalAddress : public CORBA::Address {
public:
    LocalAddress ();
    virtual ~LocalAddress ();

    virtual string stringify () const;
    virtual const char *proto () const;
    virtual CORBA::Transport *make_transport () const;
    virtual CORBA::TransportServer *make_transport_server () const;
    virtual CORBA::IORProfile *make_ior_profile (CORBA::Octet *,
						 CORBA::ULong) const;
    virtual CORBA::Boolean is_local () const;

    virtual LocalAddress *clone () const;

    virtual CORBA::Boolean operator== (const CORBA::Address &) const;
    virtual CORBA::Boolean operator< (const CORBA::Address &) const;
};

class LocalAddressParser : public CORBA::AddressParser {
public:
    LocalAddressParser ();
    virtual ~LocalAddressParser ();
    virtual LocalAddress *parse (const char *) const;
    virtual const char *proto () const;
};


class InetAddress : public CORBA::Address {
    CORBA::UShort _port;
    string _host;
    vector<CORBA::Octet> _ipaddr;

    CORBA::Boolean resolve_ip () const;
    CORBA::Boolean resolve_host () const;
public:
    InetAddress (struct sockaddr_in &sin);
    InetAddress (const char *host = NULL, CORBA::UShort port = 0);
    InetAddress (const vector<CORBA::Octet> &, CORBA::UShort port = 0);
    virtual ~InetAddress ();

    virtual string stringify () const;
    virtual const char *proto () const;
    virtual CORBA::Transport *make_transport () const;
    virtual CORBA::TransportServer *make_transport_server () const;
    virtual CORBA::IORProfile *make_ior_profile (CORBA::Octet *,
						 CORBA::ULong) const;
    virtual CORBA::Boolean is_local () const;

    virtual InetAddress *clone () const;

    virtual CORBA::Boolean operator== (const CORBA::Address &) const;
    virtual CORBA::Boolean operator< (const CORBA::Address &) const;

    CORBA::Boolean valid () const;

    const char *host () const;
    void host (const char *);

    const vector<CORBA::Octet> &ipaddr () const;
    void ipaddr (const vector<CORBA::Octet> &);

    CORBA::UShort port () const;
    void port (CORBA::UShort port);

    void sockaddr (struct sockaddr_in &);
    struct sockaddr_in sockaddr () const;

    static string hostname ();
    static CORBA::Boolean samehosts (const string &h1, const string &h2);
};

class InetAddressParser : public CORBA::AddressParser {
public:
    InetAddressParser ();
    virtual ~InetAddressParser ();
    virtual InetAddress *parse (const char *) const;
    virtual const char *proto () const;
};

#ifdef HAVE_SYS_UN_H
class UnixAddress : public CORBA::Address {
    string _filename;
public:
    UnixAddress (struct sockaddr_un &una);
    UnixAddress (const char *filename = 0);
    virtual ~UnixAddress ();

    virtual string stringify () const;
    virtual const char *proto () const;
    virtual CORBA::Transport *make_transport () const;
    virtual CORBA::TransportServer *make_transport_server () const;
    virtual CORBA::IORProfile *make_ior_profile (CORBA::Octet *,
						 CORBA::ULong) const;
    virtual CORBA::Boolean is_local () const;

    virtual UnixAddress *clone () const;

    virtual CORBA::Boolean operator== (const CORBA::Address &) const;
    virtual CORBA::Boolean operator< (const CORBA::Address &) const;

    const char *filename () const;
    void filename (const char *);

    void sockaddr (struct sockaddr_un &);
    struct sockaddr_un sockaddr () const;
};

class UnixAddressParser : public CORBA::AddressParser {
public:
    UnixAddressParser ();
    virtual ~UnixAddressParser ();
    virtual UnixAddress *parse (const char *) const;
    virtual const char *proto () const;
};
#endif // HAVE_SYS_UN_H

#endif // __mico_address_impl_h__
