// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_any_h__
#define __mico_any_h__

class Any {
    TypeCodeChecker *checker;
    DataEncoder *ec;
    DataDecoder *dc;
    TypeCode_ptr thetc;

    TypeCode_ptr tc () const;
    void tc (TypeCode_ptr);

    void free ();
    void copy (const Any &);

    void prepare_write ();
    void prepare_read ();
    Boolean copy_any (Any &, Boolean recurse = TRUE);
    Boolean compare_any (Any &);

    Any (TypeCode_ptr tc, Buffer *, Boolean release = FALSE);
    Any (TypeCode_ptr tc, DataDecoder *, DataEncoder *);

    Boolean get_short (Short &);
    Boolean get_ushort (UShort &);
    Boolean get_long (Long &);
    Boolean get_ulong (ULong &);
    Boolean get_float (Float &);
    Boolean get_double (Double &);
public:
    Any ();
    Any (const Any &);
    Any (TypeCode_ptr tc, void *value, Boolean release = FALSE);
    ~Any ();

    Any &operator= (const Any &);
    // begin-mico-extension
    Boolean operator== (const Any &) const;
    Boolean equivalent (const Any &) const;
    // end-mico-extension

    struct from_boolean {
	from_boolean (Boolean b) : val (b) {}
	Boolean val;
    };
    struct from_octet {
	from_octet (Octet o) : val (o) {}
	Octet val;
    };
    struct from_char {
	from_char (Char c) : val (c) {}
	Char val;
    };
    struct from_string {
	from_string (char *s, ULong b, Boolean nc = FALSE)
	    : val (s), bound (b), nocopy (nc) {}
	char *val;
	ULong bound;
	Boolean nocopy;
    };
    // begin-mico-extension
    struct from_object {
	from_object (Object_ptr o, const char *n) : name (n), val (o) {}
	const char *name;
	Object_ptr val;
    };
    // end-mico-extension

    Boolean operator<<= (Short);
    Boolean operator<<= (Long);
    Boolean operator<<= (UShort);
    Boolean operator<<= (ULong);
    Boolean operator<<= (Float);
    Boolean operator<<= (Double);
    Boolean operator<<= (const Any &);
    Boolean operator<<= (const char *);
    Boolean operator<<= (from_boolean);
    Boolean operator<<= (from_octet);
    Boolean operator<<= (from_char);
    Boolean operator<<= (from_string);
    // begin-mico-extension
    Boolean operator<<= (from_object);
    Boolean operator<<= (TypeCode_ptr);
    Boolean operator<<= (Context_ptr);
    Boolean operator<<= (Principal_ptr);
    // end-mico-extension

    struct to_boolean {
	to_boolean (Boolean &b) : ref (b) {}
	Boolean &ref;
    };
    struct to_octet {
	to_octet (Octet &o) : ref (o) {}
	Octet &ref;
    };
    struct to_char {
	to_char (Char &c) : ref (c) {}
	Char &ref;
    };
    struct to_string {
	to_string (char * &s, ULong b) : ref (s), bound (b) {}
	char * &ref;
	ULong bound;
    };
    struct to_object {
	to_object (Object_ptr &o, char **n = 0) : ref (o), name (n) {}
	Object_ptr &ref;
	char **name;
    };

    Boolean operator>>= (Short &) const;
    Boolean operator>>= (Long &) const;
    Boolean operator>>= (UShort &) const;
    Boolean operator>>= (ULong &) const;
    Boolean operator>>= (Float &) const;
    Boolean operator>>= (Double &) const;
    Boolean operator>>= (Any &) const;
    Boolean operator>>= (char * &) const;
    Boolean operator>>= (to_boolean) const;
    Boolean operator>>= (to_octet) const;
    Boolean operator>>= (to_char) const;
    Boolean operator>>= (to_string) const;
    Boolean operator>>= (to_object) const;
    // begin-mico-extension
    Boolean operator>>= (TypeCode_ptr &) const;
    Boolean operator>>= (Context_ptr &) const;
    Boolean operator>>= (Principal_ptr &) const;
    // end-mico-extension

    void replace (TypeCode_ptr tc, void *value, Boolean release = FALSE);

    const void *value () const;
    TypeCode_ptr type () const;
    // begin-mico-extension
    ULong length () const;
    void type (TypeCode_ptr);
    void reset ();
    void rewind ();

    void encode (DataEncoder &) const;
    Boolean decode (DataDecoder &);

    void marshal (DataEncoder &);
    Boolean demarshal (TypeCode_ptr, DataDecoder &);
    // end-mico-extension

    // begin-mico-extension
    // typesafe interface for constructed types
    Boolean struct_put_begin ();
    Boolean struct_put_end ();

    Boolean union_put_begin ();
    Boolean union_put_selection (Long idx);
    Boolean union_put_end ();

    Boolean enum_put (ULong val);
    Boolean any_put (Any &, Boolean recurse = TRUE);

    Boolean seq_put_begin (ULong len);
    Boolean seq_put_end ();

    Boolean array_put_begin ();
    Boolean array_put_end ();

    Boolean except_put_begin (const char *repoid);
    Boolean except_put_end ();


    Boolean struct_get_begin () const;
    Boolean struct_get_end () const;

    Boolean union_get_begin () const;
    Boolean union_get_selection (Long idx) const;
    Boolean union_get_end () const;

    Boolean enum_get (ULong &val) const;
    Boolean any_get (Any &, Boolean recurse = TRUE) const;

    Boolean seq_get_begin (ULong &len) const;
    Boolean seq_get_end () const;

    Boolean array_get_begin () const;
    Boolean array_get_end () const;

    Boolean except_get_begin (char *& repoid) const;
    Boolean except_get_end () const;
    // end-mico-extension

private:
    void operator<<= (unsigned char) {}
    void operator>>= (unsigned char &) const {}
};

typedef TVar<Any> Any_var;

#endif // __mico_any_h__
