// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_basic_h__
#define __mico_basic_h__

// basic types
typedef MICO_Short Short;
typedef MICO_Long Long;
typedef MICO_UShort UShort;
typedef MICO_ULong ULong;
typedef MICO_Float Float;
typedef MICO_Double Double;
typedef MICO_Char Char;
typedef MICO_Boolean Boolean;
typedef MICO_Octet Octet;

#define MAKE_WRAPPER(T)				\
class T##Wrapper {				\
    T rep;					\
public:						\
    T##Wrapper ()				\
    {						\
    }						\
    T##Wrapper (const T##Wrapper &t)		\
	: rep (t.rep)				\
    {						\
    }						\
    T##Wrapper (T t)				\
	: rep(t)				\
    {						\
    }						\
    T##Wrapper &operator= (const T##Wrapper &t)	\
    {						\
	rep = t.rep;				\
	return *this;				\
    }						\
    T##Wrapper &operator= (T t)			\
    {						\
	rep = t;				\
	return *this;				\
    }						\
    int operator== (const T##Wrapper &t) const	\
    {						\
	return rep == t.rep;			\
    }						\
    int operator!= (const T##Wrapper &t) const	\
    {						\
	return rep != t.rep;			\
    }						\
    operator T & () const      		       	\
    {						\
	return (T &)rep;			\
    }						\
};

MAKE_WRAPPER(Char);
MAKE_WRAPPER(Octet);
MAKE_WRAPPER(Boolean);

typedef ULong Flags;
typedef ULong Status;


// forwards
class Environment;
class NamedValue;
class NVList;
class ExceptionList;
class ContextList;
class Request;
class Context;
class Principal;
class TypeCode;
class TypeCodeChecker;
class BOA;
class ORB;
class RemoteBOA;
class ImplementationDef;
class ImplRepository;
class Repository;
class InterfaceDef;
class OperationDef;
class ServerRequest;
class DynamicImplementation;

class String_var;
class Object;
class Exception;
class SystemException;
class UserException;
class UnkownUserException;
class Any;

class DataEncoder;
class DataDecoder;

typedef Environment *Environment_ptr;
typedef Environment *EnvironmentRef;
typedef NamedValue *NamedValue_ptr;
typedef NamedValue *NamedValueRef;
typedef NVList *NVList_ptr;
typedef NVList *NVListRef;
typedef ExceptionList *ExceptionList_ptr;
typedef ExceptionList *ExceptionListRef;
typedef ContextList *ContextList_ptr;
typedef ContextList *ContextListRef;
typedef Request *Request_ptr;
typedef Request *RequestRef;
typedef Context *Context_ptr;
typedef Context *ContextRef;
typedef Principal *Principal_ptr;
typedef Principal *PrincipalRef;
typedef TypeCode *TypeCode_ptr;
typedef TypeCode *TypeCodeRef;
typedef ObjVar<TypeCode> TypeCode_var;
typedef BOA *BOA_ptr;
typedef BOA *BOARef;
typedef ORB *ORB_ptr;
typedef ORB *ORBRef;
typedef InterfaceDef *InterfaceDef_ptr;
// InterfaceDefRef is defined in ir.h
typedef OperationDef *OperationDef_ptr;
// OperationDefRef is defined in ir.h
typedef ServerRequest *ServerRequest_ptr;
typedef ServerRequest *ServerRequestRef;
typedef Object *Object_ptr;
typedef Object *ObjectRef;

#endif // __mico_basic_h__
