// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_boa_h__
#define __mico_boa_h__


/************************* Interfaces ******************************/

class ORBRequest;

class BOA : public ServerlessObject {
public:
    //typedef BoundedSequenceTmpl<OctetWrapper, 1024> ReferenceData;
    typedef SequenceTmpl<OctetWrapper> ReferenceData;
    typedef TSeqVar<ReferenceData> ReferenceData_var;

    virtual Object_ptr create (const ReferenceData &,
			       InterfaceDef *,
			       ImplementationDef *,
			       DynamicImplementation * = NULL,
                               const char *repoid = 0) = 0;
    // begin-mico-extension
    virtual ~BOA ();
    virtual Boolean restoring () = 0;
    virtual Object_ptr restore (Object_ptr orig,
				const ReferenceData &,
				InterfaceDef *,
				ImplementationDef *,
				DynamicImplementation *) = 0;
    virtual ORB_ptr orb () = 0;
    virtual const char *impl_name () = 0;
    virtual void save_objects () = 0;
    virtual void dispose_objects () = 0;
    virtual void answer_invoke (ULong, Object_ptr,
                                ORBRequest *,
                                ServerRequest_ptr) = 0;
    // end-mico-extension

    virtual void dispose (Object_ptr) = 0;
    virtual ReferenceData *get_id (Object_ptr) = 0;
    virtual void change_implementation (Object_ptr,
					ImplementationDef *) = 0;
    virtual Principal_ptr get_principal (Object_ptr, Environment_ptr) = 0;

    virtual void impl_is_ready (ImplementationDef *) = 0;
    virtual void deactivate_impl (ImplementationDef *) = 0;
    virtual void obj_is_ready (Object_ptr, ImplementationDef *) = 0;
    virtual void deactivate_obj (Object_ptr) = 0;

    static BOA_ptr _duplicate (BOA_ptr);
    static BOA_ptr _nil ();
};
MICO_EXPORT_DECL void release (BOA_ptr);
MICO_EXPORT_DECL Boolean is_nil (BOA_ptr);

typedef ObjVar<BOA> BOA_var;


class BOAObjectRestorer {
    typedef vector<BOAObjectRestorer *> VecRestorer;
    static VecRestorer _restorers;
public:
    static Boolean restore_object (Object_ptr);
    static Boolean bind_object (const char *repoid,
                                const SequenceTmpl<OctetWrapper> &tag);

    BOAObjectRestorer ();
    virtual ~BOAObjectRestorer ();

    virtual Boolean restore (Object_ptr);
    virtual Boolean bind (const char *repoid,
                          const SequenceTmpl<OctetWrapper> &tag);
};


#endif // __mico_boa_h__
