// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_boa_impl_h__
#define __mico_boa_impl_h__


class ObjectRecord {
    CORBA::BOA::ReferenceData _id;
    CORBA::ImplementationDef * _impl;
    CORBA::InterfaceDef * _iface;
    CORBA::Object_ptr _local_obj;
    CORBA::Object_ptr _remote_obj;
    CORBA::DynamicImplementation *_skel;
    CORBA::Boolean _save;
    CORBA::Long _invoke_count;
    // changed for CDK
    CORBA::Boolean _shut_down;
    CORBA::Boolean _persistent;
public:

    ObjectRecord (CORBA::Object_ptr local, const CORBA::BOA::ReferenceData &,
		  CORBA::InterfaceDef *, CORBA::ImplementationDef *,
		  CORBA::DynamicImplementation * = NULL);
    ObjectRecord (CORBA::Object_ptr local, CORBA::Object_ptr remote,
		  const CORBA::BOA::ReferenceData &,
		  CORBA::InterfaceDef *, CORBA::ImplementationDef *,
		  CORBA::DynamicImplementation * = NULL);
    ~ObjectRecord ();

    CORBA::BOA::ReferenceData *id ();

    CORBA::ImplementationDef *impl ();
    void impl (CORBA::ImplementationDef *);

    CORBA::InterfaceDef *iface ();
    void iface (CORBA::InterfaceDef *);

    CORBA::Object_ptr local_obj ();
    CORBA::Object_ptr remote_obj ();

    CORBA::DynamicImplementation *skel ();
    void skel (CORBA::DynamicImplementation *);

    void save (CORBA::Boolean);
    CORBA::Boolean save () const;

    void persistent (CORBA::Boolean);
    CORBA::Boolean persistent () const;

    void invoke_count (CORBA::Long);
    CORBA::Long invoke_count () const;

    void shutdown (CORBA::Boolean);
    CORBA::Boolean shutdown () const;
};


class BOAImpl : public CORBA::BOA, public CORBA::ObjectAdapter {
    typedef map<CORBA::ObjectWrapper,
	ObjectRecord *, less<CORBA::ObjectWrapper> > MapObjRec;
    typedef list<CORBA::SharedLib *> ListShlib;

    MapObjRec lobjs, robjs;
    ListShlib shlibs;
    CORBA::ORB_ptr _orb;
    CORBA::ULong theid;
    CORBA::RemoteBOA *remote;
    CORBA::Object_ptr _active_obj;
    string _impl_name;
    CORBA::Boolean _stopped;
    CORBA::Boolean _restoring;
    CORBA::Boolean _shutdown;
    CORBA::Long _invoke_count;

    CORBA::ImplementationDef *find_impl ();
    CORBA::Object_ptr find_obj ();
    void add_record (ObjectRecord *);
    ObjectRecord *get_record (CORBA::Object_ptr);
    void del_record (CORBA::Object_ptr);
    void del_all_records ();
    string unique_id ();
    CORBA::Boolean builtin_invoke (CORBA::Object_ptr,
				   CORBA::ServerRequest_ptr,
				   CORBA::Principal_ptr);
    CORBA::Boolean activate (const char *repoid);
    void shutdown_obj (CORBA::Object_ptr);
    void shutdown_impl (CORBA::ImplementationDef *);

    void restore_internal (CORBA::Object_ptr);
public:
    BOAImpl (CORBA::ORB_ptr, int &argc, char **argv);
    virtual ~BOAImpl ();

    virtual CORBA::Object_ptr create (const ReferenceData &,
				      CORBA::InterfaceDef *,
				      CORBA::ImplementationDef *,
				      CORBA::DynamicImplementation * = NULL,
				      const char *repoid = 0);
    // begin-mico-extension
    virtual CORBA::Boolean restoring ();
    virtual CORBA::Object_ptr restore (CORBA::Object_ptr,
				       const ReferenceData &,
				       CORBA::InterfaceDef *,
				       CORBA::ImplementationDef *,
				       CORBA::DynamicImplementation *);
    virtual CORBA::ORB_ptr orb ();
    virtual const char *impl_name ();
    virtual void save_objects ();
    virtual void save_object (ObjectRecord *rec);
    virtual void dispose_objects ();
    virtual void dispose_object (ObjectRecord *rec);
    virtual CORBA::Boolean load_object (ObjectRecord *rec);
    virtual void answer_invoke (MsgId, CORBA::Object_ptr,
                                CORBA::ORBRequest *,
                                CORBA::ServerRequest_ptr);
    // end-mico-extension
    virtual void dispose (CORBA::Object_ptr);
    virtual ReferenceData *get_id (CORBA::Object_ptr);
    virtual void change_implementation (CORBA::Object_ptr,
					CORBA::ImplementationDef *);
    virtual CORBA::Principal_ptr get_principal (CORBA::Object_ptr,
						CORBA::Environment_ptr);

    virtual void impl_is_ready (CORBA::ImplementationDef *);
    virtual void deactivate_impl (CORBA::ImplementationDef *);
    virtual void obj_is_ready (CORBA::Object_ptr, CORBA::ImplementationDef *);
    virtual void deactivate_obj (CORBA::Object_ptr);


    virtual const char *get_oaid () const;
    virtual CORBA::Boolean has_object (CORBA::Object_ptr);
    virtual CORBA::Boolean is_local () const;

    virtual CORBA::ImplementationDef *get_impl (CORBA::Object_ptr);
    virtual CORBA::InterfaceDef *get_iface (CORBA::Object_ptr);

    virtual CORBA::Boolean invoke (MsgId, CORBA::Object_ptr,
				   CORBA::ORBRequest *,
				   CORBA::Principal_ptr,
				   CORBA::Boolean response_exp = TRUE);
    virtual CORBA::Boolean bind (MsgId, const char *repoid,
				 const CORBA::ORB::ObjectTag &,
				 CORBA::Address *addr);
    virtual CORBA::Boolean locate (MsgId, CORBA::Object_ptr);
    virtual CORBA::Object_ptr skeleton (CORBA::Object_ptr);
    virtual void cancel (MsgId);
};

#endif // __mico_boa_impl_h__
