// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_mico_h__
#define __mico_mico_h__

    
class Buffer {
    const int MINSIZE = 128;
    const int RESIZE_THRESH = 10000;
    const int RESIZE_INCREMENT = 10000;

    Boolean readonly;
    Long rptr, wptr;
    Long ralignbase, walignbase;
    Long len;
    Octet *buf;

    Octet *alloc (ULong sz);
    Octet *realloc (Octet *, ULong osz, ULong nsz);
    void free (Octet *);
public:
    Buffer (void *);
    Buffer (ULong sz = 0);
    Buffer (const Buffer &);
    ~Buffer ();
    Buffer &operator= (const Buffer &);
    Boolean operator== (const Buffer &);

    void reset (ULong size = MINSIZE);
    void resize (ULong needed);

    Boolean rseek_rel (Long offs)
    {
	assert (rptr + offs >= 0 &&
		rptr + offs <= wptr);
	rptr += offs;
	return TRUE;
    }
    Boolean rseek_beg (ULong offs)
    {
	assert (offs <= wptr);
	rptr = offs;
	return TRUE;
    }
    Boolean rseek_end (ULong offs)
    {
	assert (len - (Long)offs >= 0 &&
		len - (Long)offs <= wptr);
	rptr = len - offs;
	return TRUE;
    }
    ULong ralign_base () const
    {
	return ralignbase;
    }
    void ralign_base (ULong b)
    {
	ralignbase = b;
    }
    ULong rpos () const
    {
	return rptr;
    }

    Boolean ralign (ULong modulo);

    ULong peek (void *, ULong len);
    ULong peek (Octet &);

    Boolean get (void *, ULong len);
    Boolean get (Octet &);
    Boolean get1 (void *);
    Boolean get2 (void *);
    Boolean get4 (void *);
    Boolean get8 (void *);

    void wseek_rel (Long offs)
    {
	assert (!readonly);
	assert (wptr + offs >= rptr &&
		wptr + offs <= len);
	wptr += offs;
    }
    void wseek_beg (ULong offs)
    {
	assert (!readonly);
	assert (offs >= rptr &&
		offs <= len);
	wptr = offs;
    }
    void wseek_end (ULong offs)
    {
	assert (!readonly);
	assert (len - (Long)offs >= rptr);
	wptr = len - offs;
    }
    ULong walign_base () const
    {
	return walignbase;
    }
    void walign_base (ULong b)
    {
	walignbase = b;
    }
    ULong wpos () const
    {
	assert (!readonly);
	return wptr;
    }

    void walign (ULong modulo);

    void replace (const void *, ULong len);
    void replace (Octet);

    void put (const void *, ULong len);
    void put (Octet);
    void put1 (const void *);
    void put2 (const void *);
    void put4 (const void *);
    void put8 (const void *);

    ULong length () const
    {
	assert (!readonly);
	return wptr - rptr;
    }
    Octet *data ()
    {
	return &buf[rptr];
    }
    const Octet *data () const
    {
	return &buf[rptr];
    }
    Octet *buffer ()
    {
	return buf;
    }
    const Octet *buffer () const
    {
	return buf;
    }
};

typedef TVar<Buffer> Buffer_var;

#endif // __mico_mico_h__
