// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_codec_impl_h__
#define __mico_codec_impl_h__


class SimpleEncoder : public CORBA::DataEncoder {
public:
    SimpleEncoder ();
    SimpleEncoder (CORBA::Buffer *, CORBA::Boolean do_free = TRUE);
    virtual ~SimpleEncoder ();

    virtual SimpleEncoder *clone () const;
    virtual SimpleEncoder *clone (CORBA::Buffer *,
				  CORBA::Boolean do_free = TRUE) const;
    virtual const char *type () const;

    virtual void put_short (CORBA::Short);
    virtual void put_ushort (CORBA::UShort);
    virtual void put_long (CORBA::Long);
    virtual void put_ulong (CORBA::ULong);
    virtual void put_float (CORBA::Float);
    virtual void put_double (CORBA::Double);
    virtual void put_char (CORBA::Char);
    virtual void put_octet (CORBA::Octet);
    virtual void put_boolean (CORBA::Boolean);
    virtual void put_string (const char *);
};


class SimpleDecoder : public CORBA::DataDecoder {
public:
    SimpleDecoder ();
    SimpleDecoder (CORBA::Buffer *b, CORBA::Boolean do_free = TRUE);
    virtual ~SimpleDecoder ();

    virtual SimpleDecoder *clone () const;
    virtual SimpleDecoder *clone (CORBA::Buffer *b,
				  CORBA::Boolean do_free = TRUE) const;
    virtual const char *type () const;

    virtual CORBA::Boolean get_short (CORBA::Short &);
    virtual CORBA::Boolean get_ushort (CORBA::UShort &);
    virtual CORBA::Boolean get_long (CORBA::Long &);
    virtual CORBA::Boolean get_ulong (CORBA::ULong &);
    virtual CORBA::Boolean get_float (CORBA::Float &);
    virtual CORBA::Boolean get_double (CORBA::Double &);
    virtual CORBA::Boolean get_char (CORBA::Char &);
    virtual CORBA::Boolean get_octet (CORBA::Octet &);
    virtual CORBA::Boolean get_boolean (CORBA::Boolean &);
    virtual CORBA::Boolean get_string (char *&);
};


class CDREncoder : public CORBA::DataEncoder {
    CORBA::ByteOrder data_bo, mach_bo;
public:
    CDREncoder ();
    CDREncoder (CORBA::Buffer *, CORBA::Boolean do_free = TRUE,
		CORBA::ByteOrder = (CORBA::ByteOrder)-1);
    virtual ~CDREncoder ();

    virtual CDREncoder *clone () const;
    virtual CDREncoder *clone (CORBA::Buffer *,
			       CORBA::Boolean do_free = TRUE) const;
    virtual const char *type () const;

    virtual void put_short (CORBA::Short);
    virtual void put_ushort (CORBA::UShort);
    virtual void put_long (CORBA::Long);
    virtual void put_ulong (CORBA::ULong);
    virtual void put_float (CORBA::Float);
    virtual void put_double (CORBA::Double);
    virtual void put_char (CORBA::Char);
    virtual void put_octet (CORBA::Octet);
    virtual void put_boolean (CORBA::Boolean);
    virtual void put_string (const char *);

    virtual CORBA::ByteOrder byteorder () const;
    virtual void byteorder (CORBA::ByteOrder);
};


class CDRDecoder : public CORBA::DataDecoder {
    CORBA::ByteOrder data_bo, mach_bo;
public:
    CDRDecoder ();
    CDRDecoder (CORBA::Buffer *b, CORBA::Boolean do_free = TRUE,
		CORBA::ByteOrder = (CORBA::ByteOrder)-1);
    virtual ~CDRDecoder ();

    virtual CDRDecoder *clone () const;
    virtual CDRDecoder *clone (CORBA::Buffer *b,
			       CORBA::Boolean do_free = TRUE) const;
    virtual const char *type () const;

    virtual CORBA::Boolean get_short (CORBA::Short &);
    virtual CORBA::Boolean get_ushort (CORBA::UShort &);
    virtual CORBA::Boolean get_long (CORBA::Long &);
    virtual CORBA::Boolean get_ulong (CORBA::ULong &);
    virtual CORBA::Boolean get_float (CORBA::Float &);
    virtual CORBA::Boolean get_double (CORBA::Double &);
    virtual CORBA::Boolean get_char (CORBA::Char &);
    virtual CORBA::Boolean get_octet (CORBA::Octet &);
    virtual CORBA::Boolean get_boolean (CORBA::Boolean &);
    virtual CORBA::Boolean get_string (char *&);

    virtual CORBA::ByteOrder byteorder () const;
    virtual void byteorder (CORBA::ByteOrder);
};

#endif // __mico_codec_impl_h__
