// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_dsi_h__
#define __mico_dsi_h__

/*
 * XXX CORBA spec says all methods may throw SystemException (18-6) ...
 * XXX is derivation from ServerlessObject allowed ?
 *
 * memory management:
 * - ServerRequest owns res and except, even dir_params (the NVList
 *   passed to params()) is freed by ServerRequest [18-6]
 * - params() must always be called by DIR exactly once [14-26]
 */
class ORBRequest;
class ServerRequest : public ServerlessObject {
    typedef CORBA::ULong MsgId;
    CORBA::BOA_ptr boa;
    MsgId msgid;
    Object_ptr obj;
    ORBRequest *req;
    NVList_ptr dir_params;
    Context_ptr context;
    Any *res;
    Exception *ex;
public:
    // begin-mico-extension
    ServerRequest (ORBRequest *req, Object_ptr obj, MsgId msgid,
                   CORBA::BOA_ptr boa);
    ~ServerRequest ();

    ORBRequest *request ();
    void set_out_args ();
    // end-mico-extension

    const char *op_name ();
    OperationDef_ptr op_def ();
    Context_ptr ctx ();
    void params (NVList_ptr params);
    void result (Any *val);
    void exception (Any *val);
    void exception (Exception *ex);
    Exception *exception ();

    static ServerRequest_ptr _duplicate (ServerRequest_ptr);
    static ServerRequest_ptr _nil ();
};
MICO_EXPORT_DECL void release (ServerRequest_ptr);
MICO_EXPORT_DECL Boolean is_nil (ServerRequest_ptr);

typedef ObjVar<ServerRequest> ServerRequest_var;


// base class for all object implementations. constructor creates
// object reference and registers with BOA.
class DynamicImplementation : virtual public Object {
    ORB_ptr orb;
    BOA_ptr boa;
public:
    // begin-mico-extension
    DynamicImplementation ();
    virtual ~DynamicImplementation ();

    virtual CORBA::Boolean _save_object ();
    ORB_ptr _orb ();
    void _create_ref (const BOA::ReferenceData &, InterfaceDef *,
		      ImplementationDef *, const char *repoid);
    void _restore_ref (Object_ptr orig, const BOA::ReferenceData &,
		       InterfaceDef *, ImplementationDef *);
    ImplementationDef *_find_impl (const char *repoid, const char *name);
    InterfaceDef *_find_iface (const char *repoid);
    // end-mico-extension
    
    BOA_ptr _boa ();
    Object_ptr _this ();
	
    virtual void invoke (ServerRequest_ptr request,
			 Environment &env) = 0;
};


#endif // __mico_dsi_h__

