// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __dynany_impl_h__
#define __dynany_impl_h__

#include <mico/dynany.h>


class DynAny_impl : virtual public DynamicAny::DynAny_skel {
protected:
    vector<DynamicAny::DynAny_var> _elements;
    CORBA::Long _index;
    CORBA::TypeCode_var _type;

    static DynamicAny::DynAnyFactory_ptr _factory();

    virtual void update_element (CORBA::Long idx);
public:
    DynAny_impl ();
    virtual ~DynAny_impl ();

    void assign (DynamicAny::DynAny_ptr dyn_any);
    DynamicAny::DynAny_ptr copy ();

    CORBA::TypeCode_ptr type ();
    void destroy ();
    void insert_boolean (CORBA::Boolean value);
    void insert_octet (CORBA::Octet value);
    void insert_char (CORBA::Char value);
    void insert_short (CORBA::Short value);
    void insert_ushort (CORBA::UShort value);
    void insert_long (CORBA::Long value);
    void insert_ulong (CORBA::ULong value);
    void insert_float (CORBA::Float value);
    void insert_double (CORBA::Double value);
    void insert_string (const char* value);
    void insert_reference (CORBA::Object_ptr value);
    void insert_typecode (CORBA::TypeCode_ptr value);
    void insert_any (const CORBA::Any &value);
    CORBA::Boolean get_boolean ();
    CORBA::Octet get_octet ();
    CORBA::Char get_char ();
    CORBA::Short get_short ();
    CORBA::UShort get_ushort ();
    CORBA::Long get_long ();
    CORBA::ULong get_ulong ();
    CORBA::Float get_float ();
    CORBA::Double get_double ();
    char* get_string ();
    CORBA::Object_ptr get_reference ();
    CORBA::TypeCode_ptr get_typecode ();
    CORBA::Any* get_any ();
    DynamicAny::DynAny_ptr current_component ();
    CORBA::Boolean next ();
    CORBA::Boolean seek (CORBA::Long index);
    void rewind ();
};


class DynBasic_impl : virtual public DynAny_impl,
		      virtual public DynamicAny::DynBasic_skel
{
private:
    CORBA::Any _value;

public:
    DynBasic_impl (CORBA::TypeCode_ptr);
    DynBasic_impl (const CORBA::Any &);
    virtual ~DynBasic_impl ();

    void destroy ();
    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();
};


/*
class DynFixed_impl : virtual public DynAny_impl,
		      virtual public ::DynamicAny::DynFixed_skel
{
public:
    DynFixed_impl (CORBA::TypeCode_ptr);
    DynFixed_impl (const CORBA::Any &);
    virtual ~DynFixed_impl ();

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    DynamicAny::OctetSeq* get_value ();
    void set_value (const DynamicAny::OctetSeq& val);
};
*/

class DynEnum_impl : virtual public DynAny_impl,
		     virtual public ::DynamicAny::DynEnum_skel
{
private:
    CORBA::Any _value;

public:
    DynEnum_impl (CORBA::TypeCode_ptr);
    DynEnum_impl (const CORBA::Any &);
    virtual ~DynEnum_impl ();

    void destroy ();
    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    char* value_as_string ();
    void value_as_string (const char* value);
    CORBA::ULong value_as_ulong ();
    void value_as_ulong (CORBA::ULong value);
};


class DynStruct_impl : virtual public DynAny_impl,
		       virtual public ::DynamicAny::DynStruct_skel
{
private:
    CORBA::Boolean _isexcept;
public:
    DynStruct_impl (CORBA::TypeCode_ptr);
    DynStruct_impl (const CORBA::Any &);
    virtual ~DynStruct_impl ();

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    DynamicAny::FieldName current_member_name ();
    DynamicAny::TCKind current_member_kind ();
    DynamicAny::NameValuePairSeq* get_members ();
    void set_members (const DynamicAny::NameValuePairSeq& value);
};


class DynUnion_impl : virtual public DynAny_impl,
		      virtual public ::DynamicAny::DynUnion_skel
{
private:
    virtual void update_element (CORBA::Long idx);
    CORBA::Long _member_idx;
public:
    DynUnion_impl (CORBA::TypeCode_ptr);
    DynUnion_impl (const CORBA::Any &);
    virtual ~DynUnion_impl ();

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::Boolean set_as_default ();
    void set_as_default (CORBA::Boolean value);
    DynamicAny::FieldName member_name ();
    void member_name (const DynamicAny::FieldName value);
    DynamicAny::DynAny_ptr discriminator ();
    DynamicAny::TCKind discriminator_kind ();
    DynamicAny::DynAny_ptr member ();
    DynamicAny::TCKind member_kind ();
};


class DynSequence_impl : virtual public DynAny_impl,
			 virtual public ::DynamicAny::DynSequence_skel
{
private:
    CORBA::ULong _length;
public:
    DynSequence_impl (CORBA::TypeCode_ptr);
    DynSequence_impl (const CORBA::Any &);
    virtual ~DynSequence_impl ();

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::ULong length ();
    void length (CORBA::ULong value);
    DynamicAny::AnySeq* get_elements ();
    void set_elements (const DynamicAny::AnySeq& value);
};


class DynArray_impl : virtual public DynAny_impl,
		      virtual public ::DynamicAny::DynArray_skel
{
private:
    CORBA::ULong _length;
public:
    DynArray_impl (CORBA::TypeCode_ptr);
    DynArray_impl (const CORBA::Any &);
    virtual ~DynArray_impl ();

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    DynamicAny::AnySeq* get_elements ();
    void set_elements (const DynamicAny::AnySeq& value);
};


class DynAnyFactory_impl : virtual public DynamicAny::DynAnyFactory_skel {
public:
    DynAnyFactory_impl ();
    virtual ~DynAnyFactory_impl ();
    DynamicAny::DynAny_ptr create_dyn_any (const CORBA::Any& value);
    DynamicAny::DynAny_ptr create_dyn_tc (CORBA::TypeCode_ptr type);
    DynamicAny::DynAny_ptr create_basic_dyn_any (CORBA::TypeCode_ptr type);
    DynamicAny::DynStruct_ptr create_dyn_struct (CORBA::TypeCode_ptr type);
    DynamicAny::DynSequence_ptr create_dyn_sequence (CORBA::TypeCode_ptr type);
    DynamicAny::DynArray_ptr create_dyn_array (CORBA::TypeCode_ptr type);
    DynamicAny::DynUnion_ptr create_dyn_union (CORBA::TypeCode_ptr type);
    DynamicAny::DynEnum_ptr create_dyn_enum (CORBA::TypeCode_ptr type);
    //DynamicAny::DynAny_ptr create_dyn_fixed (CORBA::TypeCode_ptr type);
};

#endif
