// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_except_h__
#define __mico_except_h__

#undef minor


class Exception {
    static void (*_old_terminate_handler) ();
    static void _terminate_handler ();
    static char *_last_except_desc;
public:
    Exception (const Exception &);
    virtual ~Exception ();
    virtual Exception &operator= (const Exception &);

    virtual void _print (ostream &) const;
    virtual void _encode (DataEncoder &) const;
    virtual const char *_repoid () const;
    virtual Exception *_clone () const;
    virtual void _prepare_throw () const;

    static Exception *_decode (Any &);
    static Exception *_decode (DataDecoder &);

    static void _throw_failed (Exception *ex);
    static void _init ();
protected:
    Exception ();
};

typedef ExceptVar<Exception> Exception_var;


enum CompletionStatus {
    COMPLETED_YES,
    COMPLETED_NO,
    COMPLETED_MAYBE,
};

// XXX: add T_var's for all Exceptions

class SystemException : public Exception {
    ULong _minor;
    CompletionStatus _completed;
public:
    SystemException ();
    SystemException (const SystemException &);
    SystemException (ULong minor, CompletionStatus status);
    SystemException &operator= (const SystemException &);
    virtual ~SystemException ();

    ULong minor () const;
    void minor (ULong);

    CompletionStatus completed () const;
    void completed (CompletionStatus);

    virtual void _print (ostream &) const;
    virtual void _encode (DataEncoder &) const;
    virtual const char *_repoid () const;
    virtual SystemException *_clone () const;

    static SystemException *_narrow (Exception *);
    static SystemException *_decode (Any &);
    static SystemException *_decode (DataDecoder &);

    static SystemException *_create_sysex (const char *repoid, ULong minor,
					   CompletionStatus status);
    static Boolean _is_sysex (const char *repoid);
};

typedef ExceptVar<SystemException> SystemException_var;


class UserException : public Exception {
public:
    virtual ~UserException ();

    virtual void _throwit () const;
    virtual void _print (ostream &) const;
    virtual void _encode (DataEncoder &) const;
    virtual const char *_repoid () const;
    virtual UserException *_clone () const;

    static UserException *_narrow (Exception *);
    static UserException *_decode (Any &);
    static UserException *_decode (DataDecoder &);
};

typedef ExceptVar<UserException> UserException_var;


class UnknownUserException : public UserException {
    Any *_except;
    DataDecoder *_dc;
    string _ex_repoid;
public:
    UnknownUserException (Any *);
    UnknownUserException (DataDecoder *);
    UnknownUserException ();
    UnknownUserException (const UnknownUserException &);
    UnknownUserException &operator= (const UnknownUserException &);
    virtual ~UnknownUserException ();

    Any &exception (TypeCode_ptr = 0);

    virtual void _throwit () const;
    virtual void _print (ostream &) const;
    virtual void _encode (DataEncoder &) const;
    virtual const char *_repoid () const;
    virtual const char *_except_repoid ();
    virtual UnknownUserException *_clone () const;

    static UnknownUserException *_narrow (Exception *);
};

typedef ExceptVar<UnknownUserException> UnknownUserException_var;


#define SYSEXC(name) \
class name : public SystemException { \
public: \
  name (); \
  name (const name &ex); \
  name (ULong minor, CompletionStatus status); \
  name &operator= (const name &ex); \
  virtual ~name (); \
  virtual const char *_repoid () const; \
  virtual name *_clone () const; \
  static name *_narrow (Exception *ex); \
}; \
typedef ExceptVar<name> name##_var;


SYSEXC (UNKNOWN)
SYSEXC (BAD_PARAM)
SYSEXC (NO_MEMORY)
SYSEXC (IMP_LIMIT)
SYSEXC (COMM_FAILURE)
SYSEXC (INV_OBJREF)
SYSEXC (NO_PERMISSION)
SYSEXC (INTERNAL)
SYSEXC (MARSHAL)
SYSEXC (INITIALIZE)
SYSEXC (NO_IMPLEMENT)
SYSEXC (BAD_TYPECODE)
SYSEXC (BAD_OPERATION)
SYSEXC (NO_RESOURCES)
SYSEXC (NO_RESPONSE)
SYSEXC (PERSIST_STORE)
SYSEXC (BAD_INV_ORDER)
SYSEXC (TRANSIENT)
SYSEXC (FREE_MEM)
SYSEXC (INV_IDENT)
SYSEXC (INV_FLAG)
SYSEXC (INTF_REPOS)
SYSEXC (BAD_CONTEXT)
SYSEXC (OBJ_ADAPTER)
SYSEXC (DATA_CONVERSION)
SYSEXC (OBJECT_NOT_EXIST)

#undef SYSEXC

#endif // __mico_except_h__
