// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_iop_h__
#define __mico_iop_h__

class GIOPCodec {
    CORBA::DataDecoder *dc;
    CORBA::DataEncoder *ec;
    CORBA::ULong headerlen;
    CORBA::ULong size_offset;

    CORBA::ULong put_header (CORBA::Buffer &out, CORBA::GIOP::MsgType);
    void put_size (CORBA::Buffer &out, CORBA::ULong key);
    void put_contextlist (CORBA::Buffer &out,
			  const CORBA::IOP::ServiceContextList &ctx);
    void put_objectkey (CORBA::Buffer &out, CORBA::Object_ptr obj);
    void put_args (CORBA::Buffer &out, CORBA::ORBRequest *,
		   CORBA::Boolean inp);

    CORBA::Boolean get_contextlist (CORBA::Buffer &in,
				    CORBA::IOP::ServiceContextList &ctx);
    CORBA::Boolean get_objectkey (CORBA::Buffer &in, CORBA::Object_ptr obj);
public:
    GIOPCodec (CORBA::DataDecoder *dc, CORBA::DataEncoder *ec);
    ~GIOPCodec ();

    CORBA::Boolean put_invoke_request (CORBA::Buffer &out,
				       const CORBA::IOP::ServiceContextList &,
				       CORBA::ULong req_id,
				       CORBA::Boolean response_exp,
				       CORBA::Object_ptr obj,
				       CORBA::ORBRequest *req,
				       CORBA::Principal_ptr pr);

   CORBA:: Boolean put_invoke_reply (CORBA::Buffer &out,
				     const CORBA::IOP::ServiceContextList &ctx,
				     CORBA::ULong req_id,
				     CORBA::GIOP::ReplyStatusType,
				     CORBA::Object_ptr obj,
				     CORBA::ORBRequest *req);

    CORBA::Boolean put_cancel_request (CORBA::Buffer &out,
				       CORBA::ULong req_id);

    CORBA::Boolean put_locate_request (CORBA::Buffer &out,
				       CORBA::ULong req_id,
				       CORBA::Object_ptr obj);

    CORBA::Boolean put_locate_reply (CORBA::Buffer &out,
				     CORBA::ULong req_id,
				     CORBA::GIOP::LocateStatusType,
				     CORBA::Object_ptr obj);

    CORBA::Boolean put_bind_request (CORBA::Buffer &out,
				     CORBA::ULong req_id,
				     const char *repoid,
				     const CORBA::ORB::ObjectTag &);

    CORBA::Boolean put_bind_reply (CORBA::Buffer &out,
				   CORBA::ULong req_id,
				   CORBA::GIOP::LocateStatusType,
				   CORBA::Object_ptr obj);

    CORBA::Boolean put_close_msg (CORBA::Buffer &out);

    CORBA::Boolean put_error_msg (CORBA::Buffer &out);

    CORBA::ULong header_length ();
    CORBA::Boolean get_header (CORBA::Buffer &in, CORBA::GIOP::MsgType &,
			       CORBA::ULong &sz);
    CORBA::Boolean get_size (CORBA::Buffer &in, CORBA::ULong &);

    CORBA::Boolean get_invoke_request (CORBA::Buffer_var &in,
				       CORBA::IOP::ServiceContextList &ctx,
				       CORBA::ULong &req_id,
				       CORBA::Boolean &response_exp,
				       CORBA::Object_ptr obj,
				       CORBA::ORBRequest * &req,
				       CORBA::Principal_ptr &pr);


    CORBA::Boolean get_invoke_reply (CORBA::Buffer_var &in,
				     CORBA::IOP::ServiceContextList &ctx,
				     CORBA::ULong &req_id,
				     CORBA::GIOP::ReplyStatusType &,
				     CORBA::Object_ptr &obj,
				     CORBA::ORBRequest * &req);

    CORBA::Boolean get_cancel_request (CORBA::Buffer &in,
				       CORBA::ULong &req_id);

    CORBA::Boolean get_locate_request (CORBA::Buffer &in,
				       CORBA::ULong &req_id,
				       CORBA::Object_ptr obj);

    CORBA::Boolean get_locate_reply (CORBA::Buffer &in,
				     CORBA::ULong &req_id,
				     CORBA::GIOP::LocateStatusType &,
				     CORBA::Object_ptr &obj);

    CORBA::Boolean get_bind_request (CORBA::Buffer &in,
				     char *& repoid,
				     CORBA::ORB::ObjectTag &);

    CORBA::Boolean get_bind_reply (CORBA::Buffer &in,
			    CORBA::GIOP::LocateStatusType &,
			    CORBA::Object_ptr &obj);

    CORBA::Boolean get_close_msg (CORBA::Buffer &in);

    CORBA::Boolean get_error_msg (CORBA::Buffer &in);
};


class GIOPRequest : public CORBA::ORBRequest {
    string opname;

    CORBA::DataEncoder *iec;
    CORBA::DataDecoder *idc;
    CORBA::ULong istart;

    CORBA::DataEncoder *oec;
    CORBA::DataDecoder *odc;
    CORBA::ULong ostart;

    CORBA::Boolean is_except;
public:
    GIOPRequest (const char *op, CORBA::DataDecoder *indata);
    GIOPRequest (CORBA::DataDecoder *outdata, CORBA::Boolean is_except);
    virtual ~GIOPRequest ();

    virtual const char *op_name();

    virtual void get_in_args (CORBA::NVList_ptr iparams,
			      CORBA::Context_ptr &ctx);
    virtual void get_in_args (CORBA::DataEncoder *);

    virtual void get_out_args (CORBA::Any *res, CORBA::NVList_ptr oparams,
			       CORBA::Exception *&ex);
    virtual void get_out_args (CORBA::DataEncoder *,
			       CORBA::Boolean &is_except);

    virtual void set_out_args (CORBA::Any *res, CORBA::NVList_ptr oparams);
    virtual void set_out_args (CORBA::Exception *ex);

    virtual void copy_out_args (CORBA::ORBRequest *);
    virtual void copy_in_args (CORBA::ORBRequest *);

    virtual const char *type ();
    CORBA::ByteOrder input_byteorder ();
    CORBA::ByteOrder output_byteorder ();

    static GIOPRequest* _nil ();
    static GIOPRequest* _duplicate (GIOPRequest *);
};


struct GIOPConnCallback;

class GIOPConn : public CORBA::TransportCallback,
		 public CORBA::DispatcherCallback {
    CORBA::Transport *transp;
    list<CORBA::Buffer *> outbufs;
    CORBA::Buffer *inbuf;
    CORBA::ULong msglen;
    GIOPConnCallback *cb;
    GIOPCodec *codec;
    CORBA::ORB_ptr orb;
    CORBA::Long refcnt;
    CORBA::Long idle_tmout;
    CORBA::Boolean have_tmout;
    CORBA::Boolean have_wselect;

    void check_idle ();
    void check_busy ();

    void do_read ();
    void do_write ();
public:
    GIOPConn (CORBA::ORB_ptr, CORBA::Transport *, GIOPConnCallback *,
	      GIOPCodec *, CORBA::Long tmout = 0);
    virtual ~GIOPConn ();

    virtual void callback (CORBA::Transport *,
			   CORBA::TransportCallback::Event);
    virtual void callback (CORBA::Dispatcher *,
			   CORBA::DispatcherCallback::Event);

    void ref ();
    CORBA::Boolean deref (CORBA::Boolean all = FALSE);

    void output (CORBA::Buffer *);
    CORBA::Buffer *input ();

    CORBA::Transport *transport ();
};

struct GIOPConnCallback {
    enum Event { InputReady, Closed, Idle };
    virtual void callback (GIOPConn *, Event) = 0;
    virtual ~GIOPConnCallback ();
};


class IIOPProxy : public CORBA::ObjectAdapter, public GIOPConnCallback {
    typedef map<MsgId, GIOPConn *, less<MsgId> > MapIdConn;
    typedef map<CORBA::AddressWrapper, GIOPConn *,
        less<CORBA::AddressWrapper> > MapAddrConn;
    typedef vector<CORBA::IORProfile::ProfileId> ProfIdVec;

    ProfIdVec valid_profiles;
    MapIdConn ids;
    MapAddrConn conns;
    CORBA::ORB_ptr orb;
    GIOPCodec *codec;

    GIOPConn *make_conn (CORBA::Object_ptr);
    GIOPConn *make_conn (const CORBA::Address *);
    GIOPConn *find_conn (MsgId);
    void kill_conn (GIOPConn *);
    void conn_error (GIOPConn *, CORBA::Boolean send_error = TRUE);

    void handle_input (GIOPConn *);
    void exec_invoke_reply (CORBA::Buffer *, CORBA::ULong req_id,
			    CORBA::GIOP::ReplyStatusType,
			    CORBA::Object_ptr, CORBA::ORBRequest *);
    void handle_invoke_reply (GIOPConn *conn, CORBA::Buffer_var &b);
    void handle_locate_reply (GIOPConn *conn, CORBA::Buffer *b);
public:
    IIOPProxy (CORBA::ORB_ptr);
    virtual ~IIOPProxy ();

    virtual const char *get_oaid () const;
    virtual CORBA::Boolean has_object (CORBA::Object_ptr);
    virtual CORBA::Boolean is_local () const;

    virtual CORBA::Boolean invoke (MsgId, CORBA::Object_ptr,
				   CORBA::ORBRequest *,
				   CORBA::Principal_ptr,
				   CORBA::Boolean repsonse_exp);
    virtual CORBA::Boolean bind (MsgId, const char *repoid,
				 const CORBA::ORB::ObjectTag &,
				 CORBA::Address *addr);
    virtual CORBA::Boolean locate (MsgId, CORBA::Object_ptr);
    virtual CORBA::Object_ptr skeleton (CORBA::Object_ptr);
    virtual void cancel (MsgId);

    virtual void callback (GIOPConn *, GIOPConnCallback::Event);
};


class IIOPInvokeRec {
public:
    typedef CORBA::ULong MsgId;
private:
    CORBA::ORBRequest *request;
    CORBA::Object_ptr obj;
    CORBA::Principal_ptr pr;
    MsgId orb_id, req_id;
    GIOPConn *connection;
public:
    IIOPInvokeRec (GIOPConn *, MsgId reqid, MsgId orbid,
		   CORBA::ORBRequest *, CORBA::Object_ptr,
		   CORBA::Principal_ptr);
    IIOPInvokeRec (GIOPConn *, MsgId reqid, MsgId orbid, CORBA::Object_ptr);
    ~IIOPInvokeRec ();

    CORBA::ORBRequest *req();
    MsgId orbid();
    void orbid (MsgId);
    MsgId reqid();
    GIOPConn *conn();
};

class IIOPServer : public CORBA::ORBCallback, public GIOPConnCallback,
		   public CORBA::TransportServerCallback {
    typedef CORBA::ULong MsgId;
    typedef map<MsgId, IIOPInvokeRec *, less<MsgId> > MapIdConn;
    typedef list<GIOPConn *> ListConn;
    typedef vector<CORBA::TransportServer *> VecTranspServ;

    VecTranspServ tservers;
    ListConn conns;
    MapIdConn orbids;
    CORBA::ORB_ptr orb;
    GIOPCodec *codec;

    IIOPInvokeRec *get_invoke_reqid (MsgId, GIOPConn *conn);
    IIOPInvokeRec *get_invoke_orbid (MsgId);
    void add_invoke (IIOPInvokeRec *);
    void del_invoke_reqid (MsgId, GIOPConn *conn);
    void del_invoke_orbid (MsgId);

    void kill_conn (GIOPConn *);
    void conn_error (GIOPConn *, CORBA::Boolean send_error = TRUE);

    void handle_input (GIOPConn *);
    MsgId exec_invoke_request (CORBA::Buffer *, CORBA::Object_ptr,
			       CORBA::ORBRequest *,
			       CORBA::Principal_ptr, CORBA::Boolean resp_exp);
    void handle_invoke_request (GIOPConn *conn, CORBA::Buffer_var &b);
    void handle_locate_request (GIOPConn *conn, CORBA::Buffer *b);
    void handle_cancel_request (GIOPConn *conn, CORBA::Buffer *b);

    void handle_invoke_reply (MsgId msgid);
    void handle_locate_reply (MsgId msgid);
    void handle_bind_reply (MsgId msgid);
public:
    IIOPServer (CORBA::ORB_ptr);
    virtual ~IIOPServer ();

    CORBA::Boolean listen (CORBA::Address *);
    CORBA::Boolean listen ();

    virtual void callback (CORBA::ORB_ptr, MsgId, CORBA::ORBCallback::Event);
    virtual void callback (GIOPConn *, GIOPConnCallback::Event);
    virtual void callback (CORBA::TransportServer *,
			   CORBA::TransportServerCallback::Event);
};

#endif // __mico_iop_h__
