// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_ioptypes_h__
#define __mico_ioptypes_h__

// mapped by hand from ioptypes.idl

struct IOP {
    typedef CORBA::ULong ServiceID;

    struct ServiceContext {
	ServiceContext ();
	ServiceContext (const ServiceContext &);
	ServiceContext &operator= (const ServiceContext &);
	~ServiceContext ();

        ServiceID context_id;
        SequenceTmpl<CORBA::OctetWrapper> context_data;
    };
    typedef SequenceTmpl<ServiceContext> ServiceContextList;

    const ServiceID TransactionService = 0;
#if 0    
    typedef CORBA::ULong ProfileId;
    const ProfileId TAG_INTERNET_IOP = 0;
    const ProfileId TAG_MULTIPLE_COMPONENTS = 1;

    struct TaggedProfile {
        ProfileId tag;
        SequenceTmpl<CORBA::OctetWrapper> profile_data;
    };

    struct IOR {
        CORBA::String_var type_id;
        SequenceTmpl<TaggedProfile> profiles;
    };

    typedef CORBA::ULong ComponentId;
    struct TaggedComponent {
        ComponentId tag;
        SequenceTmpl<CORBA::OctetWrapper> component_data;
    };
    typedef SequenceTmpl<TaggedComponent> MultipleComponentProfile;
#endif
};


struct GIOP {
    enum MsgType {
        Request, Reply, CancelRequest,
        LocateRequest, LocateReply,
        CloseConnection, MessageError
    };
    
    enum ReplyStatusType {
        NO_EXCEPTION, USER_EXCEPTION,
        SYSTEM_EXCEPTION, LOCATION_FORWARD
    };

    enum LocateStatusType {
        UNKNOWN_OBJECT,
        OBJECT_HERE,
        OBJECT_FORWARD
    };
#if 0
    struct Version {
        CORBA::Char major;
        CORBA::Char minor;
    };

    struct MessageHeader {
        CORBA::Char magic[4];
        Version GIOP_version;
        CORBA::Boolean byte_order;
        CORBA::Octet message_type;
        CORBA::ULong message_size;
    };

    struct RequestHeader {
        IOP::ServiceContextList service_context;
        CORBA::ULong request_id;
        CORBA::Boolean response_expected;
        SequenceTmpl<CORBA::OctetWrapper> object_key;
        CORBA::String_var operation;
        CORBA::Principal requesting_principal;
    };

    struct ReplyHeader {
        IOP::ServiceContextList service_context;
        CORBA::ULong request_id;
        ReplyStatusType reply_status;
    };

    struct CancelRequestHeader {
        CORBA::ULong request_id;
    };

    struct LocateRequestHeader {
        CORBA::ULong request_id;
        SequenceTmpl<CORBA::OctetWrapper> object_key;
    };

    struct LocateReplyHeader {
        CORBA::ULong request_id;
        LocateStatusType locate_status;
    };
#endif
};

#if 0
struct IIOP {
    struct Version {
        CORBA::Char major;
        CORBA::Char minor;
    };

    struct ProfileBody {
        Version iiop_version;
        CORBA::String_var host;
        CORBA::UShort port;
        SequenceTmpl<CORBA::OctetWrapper> object_key;
    };
};
#endif

#endif // __mico_ioptypes_h__
