// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_ior_h__
#define __mico_ior_h__


/************************* ForwardDecls ******************************/


class IOR;
class IORProfile;
class IORProfileDecoder;
class DataDecoder;
class DataEncoder;
class Address;


/************************** Interfaces *******************************/


class IORProfile {
    static vector<IORProfileDecoder *> decoders;
protected:
    void copy (IORProfile *);
public:
    enum ProfileId {
	TAG_INTERNET_IOP = 0,
	TAG_MULTIPLE_COMPONENTS = 1,
	TAG_LOCAL = 20000,    // mico-extension
	TAG_ANY = 20001,      // mico-extension
	TAG_UNIX_IOP = 20002, // mico-extension
    };

    static IORProfile *decode (DataDecoder &);
    static void register_decoder (IORProfileDecoder *);
    static void unregister_decoder (IORProfileDecoder *);

    virtual void encode (DataEncoder &) const = 0;
    virtual const Address *addr () const = 0;
    virtual ProfileId id () const = 0;
    virtual void objectkey (Octet *, Long length) = 0;
    virtual const Octet *objectkey (Long &length) const = 0;
    virtual Boolean reachable () = 0;

    virtual IORProfile *clone () const = 0;
    virtual Boolean operator== (const IORProfile &) const = 0;
    virtual Boolean operator< (const IORProfile &) const = 0;

    virtual ~IORProfile ();
};

class IORProfileDecoder {
public:
    typedef IORProfile::ProfileId ProfileId;

    virtual IORProfile *decode (DataDecoder &) const = 0;
    virtual ProfileId id () const = 0;

    virtual ~IORProfileDecoder ();
};

class IOR {
public:
    typedef vector<IORProfile *> IORProfileVec;
private:
    string oid;
    IORProfileVec tags;

    void free ();
    void copy (const IOR &);
public:
    IOR ();
    IOR (const char *objid, const IORProfileVec &tags);
    IOR (DataDecoder &);
    IOR (const char *); // uses CDRDecoder
    IOR (const IOR &);
    ~IOR ();
    IOR &operator= (const IOR &);
    Boolean from_string (const char *);
    Boolean operator== (const IOR &) const;
    Boolean operator< (const IOR &) const;

    const Address *addr (IORProfile::ProfileId = IORProfile::TAG_ANY,
			 Boolean find_unreachable = FALSE) const;
    IORProfile *profile (IORProfile::ProfileId = IORProfile::TAG_ANY,
			 Boolean find_unreachable = FALSE);
    void add_profile (IORProfile *);
    const char *objid () const;
    void objid (const char *);
    void objectkey (Octet *, ULong len);
    void encode (DataEncoder &) const;
    Boolean decode (DataDecoder &);
    string stringify () const; // uses CDREncoder
};


#endif // __mico_ior_h__
