// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_ior_impl_h__
#define __mico_ior_impl_h__


class LocalProfile : public CORBA::IORProfile {
    string host;
    CORBA::Long pid;
    CORBA::Octet *objkey;
    CORBA::ULong length;
    LocalAddress myaddr;
    ProfileId tagid;
public:
    LocalProfile (CORBA::Octet *objkey, CORBA::ULong length,
		  ProfileId = TAG_LOCAL,
		  const char *host = 0, CORBA::Long pid = -1);
    LocalProfile (const LocalProfile &);
    virtual ~LocalProfile ();
    LocalProfile &operator= (const LocalProfile &);

    virtual void encode (CORBA::DataEncoder &) const;
    virtual const CORBA::Address *addr () const;
    virtual ProfileId id () const;
    virtual void objectkey (CORBA::Octet *, CORBA::Long length);
    virtual const CORBA::Octet *objectkey (CORBA::Long &length) const;
    virtual CORBA::Boolean reachable ();

    virtual LocalProfile *clone () const;
    virtual CORBA::Boolean operator== (const CORBA::IORProfile &) const;
    virtual CORBA::Boolean operator< (const CORBA::IORProfile &) const;
};

class LocalProfileDecoder : public CORBA::IORProfileDecoder {
    CORBA::IORProfile::ProfileId tagid;
public:
    LocalProfileDecoder (CORBA::IORProfile::ProfileId
			 = CORBA::IORProfile::TAG_LOCAL);
    virtual ~LocalProfileDecoder ();
    virtual LocalProfile *decode (CORBA::DataDecoder &) const;
    virtual ProfileId id () const;
};


class IIOPProfile : public CORBA::IORProfile {
    CORBA::Octet *objkey;
    CORBA::ULong length;
    InetAddress myaddr;
    ProfileId tagid;
public:
    IIOPProfile (CORBA::Octet *objkey, CORBA::ULong length,
		 const InetAddress &, ProfileId = TAG_INTERNET_IOP);
    IIOPProfile (const IIOPProfile &);
    virtual ~IIOPProfile ();
    IIOPProfile &operator= (const IIOPProfile &);

    virtual void encode (CORBA::DataEncoder &) const;
    virtual const CORBA::Address *addr () const;
    virtual ProfileId id () const;
    virtual void objectkey (CORBA::Octet *, CORBA::Long length);
    virtual const CORBA::Octet *objectkey (CORBA::Long &length) const;
    virtual CORBA::Boolean reachable ();

    virtual IIOPProfile *clone () const;
    virtual CORBA::Boolean operator== (const CORBA::IORProfile &) const;
    virtual CORBA::Boolean operator< (const CORBA::IORProfile &) const;
};

class IIOPProfileDecoder : public CORBA::IORProfileDecoder {
    CORBA::IORProfile::ProfileId tagid;
public:
    IIOPProfileDecoder (CORBA::IORProfile::ProfileId
			= CORBA::IORProfile::TAG_INTERNET_IOP);
    virtual ~IIOPProfileDecoder ();
    virtual IIOPProfile *decode (CORBA::DataDecoder &) const;
    virtual ProfileId id () const;
};

#ifdef HAVE_SYS_UN_H
class UIOPProfile : public CORBA::IORProfile {
    string host;
    CORBA::Octet *objkey;
    CORBA::ULong length;
    UnixAddress myaddr;
    ProfileId tagid;
public:
    UIOPProfile (CORBA::Octet *objkey, CORBA::ULong length,
		 const UnixAddress &,
		 ProfileId = TAG_UNIX_IOP,
		 const char *host = 0);
    UIOPProfile (const UIOPProfile &);
    virtual ~UIOPProfile ();
    UIOPProfile &operator= (const UIOPProfile &);

    virtual void encode (CORBA::DataEncoder &) const;
    virtual const CORBA::Address *addr () const;
    virtual ProfileId id () const;
    virtual void objectkey (CORBA::Octet *, CORBA::Long length);
    virtual const CORBA::Octet *objectkey (CORBA::Long &length) const;
    virtual CORBA::Boolean reachable ();

    virtual UIOPProfile *clone () const;
    virtual CORBA::Boolean operator== (const CORBA::IORProfile &) const;
    virtual CORBA::Boolean operator< (const CORBA::IORProfile &) const;
};

class UIOPProfileDecoder : public CORBA::IORProfileDecoder {
    CORBA::IORProfile::ProfileId tagid;
public:
    UIOPProfileDecoder (CORBA::IORProfile::ProfileId
			= CORBA::IORProfile::TAG_UNIX_IOP);
    virtual ~UIOPProfileDecoder ();
    virtual UIOPProfile *decode (CORBA::DataDecoder &) const;
    virtual ProfileId id () const;
};
#endif // HAVE_SYS_UN_H

class UnknownProfile : public CORBA::IORProfile {
    ProfileId tagid;
    vector<CORBA::Octet> tagdata;
public:
    UnknownProfile (ProfileId id);
    UnknownProfile (const UnknownProfile &);
    virtual ~UnknownProfile ();
    UnknownProfile &operator= (const UnknownProfile &);

    virtual void encode (CORBA::DataEncoder &) const;
    virtual const CORBA::Address *addr () const;
    virtual ProfileId id () const;
    virtual void objectkey (CORBA::Octet *, CORBA::Long length);
    virtual const CORBA::Octet *objectkey (CORBA::Long &length) const;
    virtual CORBA::Boolean reachable ();

    virtual UnknownProfile *clone () const;
    virtual CORBA::Boolean operator== (const CORBA::IORProfile &) const;
    virtual CORBA::Boolean operator< (const CORBA::IORProfile &) const;

    static UnknownProfile *decode (CORBA::DataDecoder &,
				   ProfileId id, CORBA::ULong len);
};

#endif // __mico_ior_impl_h__
