// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_object_h__
#define __mico_object_h__

// base class for all serverless objects
class ServerlessObject {
    Long refs;
    ULong magic;
protected:
    ServerlessObject ();
    ServerlessObject (const ServerlessObject &);
    ServerlessObject &operator= (const ServerlessObject &);
    ~ServerlessObject ();
public:
    void _init ();
    void _check() const;
    void _ref();
    Boolean _deref();
    Long _refcnt() const;
};


// object, base class for all serverbased objects
class Object : public ServerlessObject {
    IOR *ior;
    ORB_ptr orb;
    string ident;
public:
    ImplementationDef *_get_implementation ();
    InterfaceDef *_get_interface ();

    Status _create_request (Context_ptr ctx,
			    const char *operation,
			    NVList_ptr arg_list,
			    NamedValue_ptr result,
			    Request_ptr &request,
			    Flags req_flags);

    Status _create_request (Context_ptr ctx,
			    const char *operation,
			    NVList_ptr arg_list,
			    NamedValue_ptr result,
			    ExceptionList_ptr,
			    ContextList_ptr,
			    Request_ptr &request,
			    Flags req_flags);

    Request_ptr _request (const char *operation);

    // begin-not-in-appendix
    Boolean _is_a (const char *logical_type_id);
    Boolean _is_a_remote (const char *logical_type_id);
    Boolean _non_existent ();
    Boolean _is_equivalent (Object_ptr);
    ULong _hash (ULong max);
    // end-not-in-appendix

    // begin-mico-extension
    Object (IOR * = 0);
    Object (const Object &);
    Object &operator= (const Object &);
    virtual ~Object ();

    ORB_ptr _orb ();
    IOR *_ior ();
    const char *_repoid ();
    const char *_ident ();
    virtual void *_narrow_helper (const char *repoid);
    // end-mico-extension

    static Object_ptr _duplicate (Object_ptr);
    static Object_ptr _nil ();
};

typedef ObjVar<Object> Object_var;

MICO_EXPORT_DECL void release (Object_ptr);
MICO_EXPORT_DECL Boolean is_nil (Object_ptr);


// needed for set's of Object_ptr and map's with Object_ptr key
class ObjectWrapper {
    Object_ptr obj;
public:
    ObjectWrapper ();
    ObjectWrapper (Object_ptr o);
    ~ObjectWrapper ();
    Boolean operator== (const ObjectWrapper &w) const;
    Boolean operator< (const ObjectWrapper &w) const;
};

// Generic prototype for narrow helper functions
typedef bool (*Narrow_proto) (Object_ptr);

#endif // __mico_object_h__
