// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_sequence_h__
#define __mico_sequence_h__


// XXX templates must be defined at scope :: due to gcc 2.7.2 bugs...
// XXX does not work for string and objectref sequences

#if 0

template<class T>
class TWrapper {
public:
    static T *create (T * = 0);
    static T *create (MICO_ULong n);
    static void destroy (T *, MICO_Boolean vec = FALSE);
};

template<class T>
class SequenceTmpl {
public:
  typedef T ElementType; // Needed in TSeqVar (see var.h)
private:
  vector<T*> vec;

  void del()
  {
      for( int i = 0; i < vec.size(); i++ )
	  TWrapper<T>::destroy (vec[i]);
      vec.erase (vec.begin(), vec.end());
  }
  void copy( const SequenceTmpl<T> &s )
  {
      for( int i = 0; i < s.vec.size(); i++ )
	  vec.push_back (TWrapper<T>::create (s.vec[i]));
  }

public:
  SequenceTmpl () {}
  SequenceTmpl (MICO_ULong max)
  {
      vec.reserve (max);
  }
  SequenceTmpl (MICO_ULong max, MICO_ULong length, T *value,
		MICO_Boolean rel = TRUE)
  {
      assert (length <= max);
      vec.reserve (max);
      for (MICO_ULong i = 0; i < length; ++i)
	  vec.push_back (TWrapper<T>::create (&value[i]));
      if (rel)
	  freebuf (value);
  }
  
  SequenceTmpl (const SequenceTmpl<T> &s)
  {
      //vec = s.vec;
      copy( s );
  }
  
  ~SequenceTmpl ()
  {
      del();
  }
  
  SequenceTmpl<T> &operator= (const SequenceTmpl<T> &s)
  {
      //vec = s.vec;
      del();
      copy( s );
      return *this;
  }
  
  MICO_ULong maximum () const
  {
      return vec.capacity ();
  }
  
  void length (MICO_ULong l)
  {
      if (l < vec.size ()) {
	  for( int i = l; i < vec.size(); i++ )
	      TWrapper<T>::destroy (vec[i]);
	  vec.erase (vec.begin() + l, vec.end());
      } else if (l > vec.size()) {
	  int s = vec.size();
	  for( int i = 0; i < l - s; i++ )
	      vec.push_back( TWrapper<T>::create() );
      }
  }
  
  MICO_ULong length () const
  {
      return vec.size ();
  }
  
  T &operator[] (MICO_ULong idx)
  {
      assert (idx < vec.size());
      return *vec[idx];
  }
  
  const T &operator[] (MICO_ULong idx) const
  {
      assert (idx < vec.size());
      return *vec[idx];
  }
  
  static T *allocbuf (MICO_ULong len)
  {
      return TWrapper<T>::create (len);
  }

  static void freebuf (T *b)
  {
      TWrapper<T>::destroy (b, TRUE);
  }
};

#else

/*
 * C++ template for unbounded sequences. The element type of the sequence
 * can be of any type except for arrays and recursive types (see SequenceIndTmpl)
 */
template<class T>
class SequenceTmpl {
public:
    typedef T ElementType; // Needed in TSeqVar (see var.h)
private:
    vector<T> vec;
public:
    SequenceTmpl () {}
    SequenceTmpl (MICO_ULong max)
    {
	vec.reserve (max);
    }
    SequenceTmpl (MICO_ULong max, MICO_ULong length, T *value,
		  MICO_Boolean rel = TRUE)
    {
	assert (length <= max);
	vec.reserve (max);
	for (MICO_ULong i = 0; i < length; ++i)
	    vec.push_back (value[i]);
	if (rel)
	    freebuf (value);
    }

    SequenceTmpl (const SequenceTmpl<T> &s)
    {
	vec = s.vec;
    }
    
    ~SequenceTmpl ()
    {
    }
    
    SequenceTmpl<T> &operator= (const SequenceTmpl<T> &s)
    {
	vec = s.vec;
	return *this;
    }
    
    MICO_ULong maximum () const
    {
	return vec.capacity ();
    }

    void length (MICO_ULong l)
    {
	if (l < vec.size ()) {
	    vec.erase (vec.begin() + l, vec.end());
	} else if (l > vec.size()) {
	    T t;
            // the (long) cast is needed for SGI STL
	    vec.insert (vec.end(), long(l - vec.size()), t);
	}
    }

    MICO_ULong length () const
    {
	return vec.size ();
    }

    T &operator[] (MICO_ULong idx)
    {
	assert (idx < vec.size());
	return vec[idx];
    }
    
    const T &operator[] (MICO_ULong idx) const
    {
	assert (idx < vec.size());
	return vec[idx];
    }

    static T *allocbuf (MICO_ULong len)
    {
	return new T[len];
    }

    static void freebuf (T *b)
    {
	delete[] b;
    }
};
#endif

template<class T>
MICO_Boolean
operator== (const SequenceTmpl<T> &v1, const SequenceTmpl<T> &v2)
{
    if (v1.length() != v2.length())
	return FALSE;
    for (long _i = 0; _i < v1.length(); ++_i) {
	if (!(v1[_i] == v2[_i]))
	    return FALSE;
    }
    return TRUE;
}

/*
 * Same as SequenceTmpl except that this template is for bounded sequences.
 */
template<class T, int max>
class BoundedSequenceTmpl {
public:
    typedef T ElementType; // Needed in TSeqVar (see var.h)
private:
    vector<T> vec;
public:
    BoundedSequenceTmpl ()
    {
	vec.reserve (max);
    }
    BoundedSequenceTmpl (MICO_ULong length, T *value,
			 MICO_Boolean rel = TRUE)
    {
	assert (length <= max);
	vec.reserve (max);
	for (MICO_ULong i = 0; i < length; ++i)
	    vec.push_back (value[i]);
	if (rel)
	    freebuf (value);
    }
    
    BoundedSequenceTmpl (const BoundedSequenceTmpl<T, max> &s)
    {
	vec = s.vec;
    }

    ~BoundedSequenceTmpl ()
    {
    }

    BoundedSequenceTmpl<T, max> &operator=
	(const BoundedSequenceTmpl<T, max> &s)
    {
	vec = s.vec;
	return *this;
    }

    MICO_ULong maximum () const
    {
	return max;
    }

    void length (MICO_ULong l)
    {
	assert (l <= max);
	if (l < vec.size ()) {
	    vec.erase (vec.begin() + l, vec.end());
	} else if (l > vec.size()) {
	    T t;
	    vec.insert (vec.end(), long(l - vec.size()), t);
	}
    }

    MICO_ULong length () const
    {
	return vec.size ();
    }

    T &operator[] (MICO_ULong idx)
    {
	assert (idx < vec.size());
	return vec[idx];
    }

    const T &operator[] (MICO_ULong idx) const
    {
	assert (idx < vec.size());
	return vec[idx];
    }

    static T *allocbuf (MICO_ULong len)
    {
	return new T[len];
    }

    static void freebuf (T *b)
    {
	delete[] b;
    }
};

template<class T, int max>
MICO_Boolean
operator== (const BoundedSequenceTmpl<T,max> &v1,
	    const BoundedSequenceTmpl<T,max> &v2)
{
    if (v1.length() != v2.length())
	return FALSE;
    for (long _i = 0; _i < v1.length(); ++_i) {
	if (!(v1[_i] == v2[_i]))
	    return FALSE;
    }
    return TRUE;
}

/*
 * C++ template for sequences of arrays and recursive types. The difference
 * is, that this template maintains it members only indirectly through
 * pointers. This is especially necessary to break infinite recursion
 * for recursive types. If this template is used for arrays, then T_elem
 * denotes the element type of the array, T the array type itself and
 * n the total number of array elements. If the template is used for
 * recursive types then T_elem and T are the same and n should be set to 1.
 */
template<class T_elem, class T, MICO_ULong n>
class SequenceIndTmpl {
public:
  typedef T ElementType; // Needed in TSeqVar (see var.h)
private:
  vector<T_elem*> vec;
public:
  SequenceIndTmpl () {}
  SequenceIndTmpl (MICO_ULong max)
  {
    vec.reserve (max);
  }
  SequenceIndTmpl (MICO_ULong max, MICO_ULong length, T* value,
		   MICO_Boolean rel = TRUE);

  SequenceIndTmpl (const SequenceIndTmpl<T_elem,T,n> &s);
    
  ~SequenceIndTmpl ();
  
  SequenceIndTmpl<T_elem,T,n>& 
  operator= (const SequenceIndTmpl<T_elem,T,n> &s);
  
  MICO_ULong maximum () const
  {
    return vec.capacity ();
  }
  
  void length (MICO_ULong l);
  
  MICO_ULong length () const
  {
    return vec.size ();
  }
  
  T& operator[] (MICO_ULong idx)
  {
    assert (idx < vec.size());
    return (T&) *vec[idx];
  }
  
  const T& operator[] (MICO_ULong idx) const
  {
    assert (idx < vec.size());
    return (T&) *vec[idx];
  }
  
  static T *allocbuf (MICO_ULong len);
  
  static void freebuf( T* b );
  
};

template<class T_elem, class T, MICO_ULong n>
MICO_Boolean
operator== (const SequenceIndTmpl<T_elem,T,n> &v1,
	    const SequenceIndTmpl<T_elem,T,n> &v2)
{
  if( v1.length() != v2.length() )
    return FALSE;
  for( long i = 0; i < v1.length(); i++ ) {
    for( long j = 0; j < n; j++ ) {
      T_elem e1 = ((T_elem*) v1[ i ])[ j ];
      T_elem e2 = ((T_elem*) v2[ i ])[ j ];
      if( !(e1 == e2) )
	return FALSE;
    }
  }
  return TRUE;
}

/*
 * Same as SequenceIndTmpl except that this one is for bounded sequences.
 */
template<class T_elem, class T, MICO_ULong n, int max>
class BoundedSequenceIndTmpl {
public:
  typedef T ElementType; // Needed in TSeqVar (see var.h)
private:
  vector<T_elem*> vec;
public:
  BoundedSequenceIndTmpl ()
  {
    vec.reserve (max);
  }
  BoundedSequenceIndTmpl (MICO_ULong length, T *value,
			  MICO_Boolean rel = TRUE);
  
  BoundedSequenceIndTmpl
  (const BoundedSequenceIndTmpl<T_elem,T,n,max> &s);
  
  ~BoundedSequenceIndTmpl ();
  
  BoundedSequenceIndTmpl<T_elem,T,n, max> &operator=
  (const BoundedSequenceIndTmpl<T_elem,T,n, max> &s);
  
  MICO_ULong maximum () const
  {
    return max;
  }
  
  void length (MICO_ULong l);
  
  MICO_ULong length () const
  {
    return vec.size ();
  }
  
  T &operator[] (MICO_ULong idx)
  {
    assert (idx < vec.size());
    return (T&) *vec[idx];
  }
  
  const T &operator[] (MICO_ULong idx) const
  {
    assert (idx < vec.size());
    return (T&) *vec[idx];
  }
  
  static T *allocbuf (MICO_ULong len);
  
  static void freebuf (T *b);
};

template<class T_elem, class T, MICO_ULong n, int max>
MICO_Boolean
operator== (const BoundedSequenceIndTmpl<T_elem,T,n,max> &v1,
	    const BoundedSequenceIndTmpl<T_elem,T,n,max> &v2)
{
  if( v1.length() != v2.length() )
    return FALSE;
  for( long i = 0; i < v1.length(); i++ ) {
    for( long j = 0; j < n; j++ ) {
      T_elem e1 = ((T_elem*) v1[ i ])[ j ];
      T_elem e2 = ((T_elem*) v2[ i ])[ j ];
      if( !(e1 == e2) )
	return FALSE;
    }
  }
  return TRUE;
}


#endif // __mico_sequence_h__
