// -*- c++ -*-
/*
 *  MICO --- a CORBA 2.0 implementation
 *  Copyright (C) 1997 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_util_h__
#define __mico_util_h__

#include <ctype.h>
#include <iostream.h>
#include <fstream.h>
#include <CORBA.h>


class MICODebug {
    CORBA::ULong _level;
    ostream *_out;
    ostream *_null;
    static MICODebug *_instance;
public:
    MICODebug (const string &file, CORBA::ULong level = 0);
    MICODebug (CORBA::ULong level = 0);
    ~MICODebug ();

    void level (CORBA::ULong);
    CORBA::ULong level () const;

    ostream &printer ();
    ostream &debugger ();
    ostream &tracer ();
    ostream &dumper ();

    static MICODebug *instance ();
};


class MICOGetOpt {
public:
    typedef map<string, string, less<string> > OptMap;
    typedef vector<pair<string,string> > OptVec;
private:
    OptMap _in_opts;
    OptVec _out_opts;

    CORBA::Boolean parse (const vector<string> &args, vector<int> &erase,
			  CORBA::Boolean ignore = FALSE);
public:
    MICOGetOpt (const OptMap &opts);
    ~MICOGetOpt ();

    CORBA::Boolean parse (int &argc, char *argv[],
			  CORBA::Boolean ignore = FALSE);
    CORBA::Boolean parse (const vector<string> &args,
			  CORBA::Boolean ignore = FALSE);
    CORBA::Boolean parse (const string &filename,
			  CORBA::Boolean ignore = FALSE);

    const OptVec &opts () const;
};


static inline CORBA::Octet
from_xdigit (CORBA::Octet c)
{
    c = tolower (c);
    assert (isxdigit (c));
    return isdigit (c) ? c - '0' : c - 'a' + 10;
}

static inline CORBA::Octet
to_xdigit (CORBA::Octet o)
{
    static const char *xdigits = "0123456789abcdef";
    assert (o < 16);
    return xdigits[o];
}


extern void mico_ieee2float (CORBA::Octet ieee[4], CORBA::Float &f);
extern void mico_float2ieee (CORBA::Octet ieee[4], CORBA::Float  f);
extern void mico_ieee2double (CORBA::Octet ieee[8], CORBA::Double &f);
extern void mico_double2ieee (CORBA::Octet ieee[8], CORBA::Double  f);
extern CORBA::Boolean mico_repoid_match (const char *repoid, const char *patt);

extern const char *xstrerror (int);
extern const char *xdec (int);
extern const char *xdec (long);
extern const char *xdec (unsigned int);
extern const char *xdec (unsigned long);

#ifndef HAVE_STRERROR_PROTO
extern "C" char *strerror (int);
#endif

#ifndef HAVE_GETHOSTNAME_PROTO
extern "C" int gethostname (char *, size_t);
#endif

#ifndef HAVE_FINITE_PROTO
extern "C" int finite (double);
#endif

#endif // __mico_util_h__
